@@ -20,197 +20,49 @@ import inspect
 from collections.abc import Mapping, Sequence
 from copy import deepcopy
 
-from more_itertools import first
-import libsonata
 import numpy as np
 import pandas as pd
-
 from cached_property import cached_property
+from more_itertools import first
 
-from bluepysnap import query
-from bluepysnap.network import NetworkObject
-from bluepysnap import utils
-from bluepysnap.exceptions import BluepySnapError
-from bluepysnap.sonata_constants import (DEFAULT_NODE_TYPE, DYNAMICS_PREFIX, Node, ConstContainer)
-from bluepysnap.circuit_ids import CircuitNodeId, CircuitNodeIds
+from bluepysnap import query, utils
 from bluepysnap._doctools import AbstractDocSubstitutionMeta
+from bluepysnap.circuit_ids import CircuitNodeId, CircuitNodeIds
+from bluepysnap.exceptions import BluepySnapError
+from bluepysnap.network import NetworkObject
+from bluepysnap.sonata_constants import DYNAMICS_PREFIX, ConstContainer, Node
 
 
-class Nodes(NetworkObject, metaclass=AbstractDocSubstitutionMeta,
-            source_word="NetworkObject", target_word="Node"):
-    """The top level Nodes accessor."""
+class NodePopulation:
+    """Node population access."""
 
-    def __init__(self, circuit):  # pylint: disable=useless-super-delegation
-        """Initialize the top level Nodes accessor."""
-        super().__init__(circuit)
-
-    def _collect_populations(self):
-        return self._get_populations(NodeStorage, self._config['networks']['nodes'])
-
-    def property_values(self, prop):
-        """Returns all the values for a given Nodes property."""
-        return set(value for pop in self.values() if prop in pop.property_names for value in
-                   pop.property_values(prop))
-
-    def ids(self, group=None, sample=None, limit=None):
-        """Returns the CircuitNodeIds corresponding to the nodes from ``group``.
+    def __init__(self, circuit, population_name):
+        """Initializes a NodePopulation object.
 
         Args:
-            group (CircuitNodeId/CircuitNodeIds/int/sequence/str/mapping/None): Which IDs will be
-            returned depends on the type of the ``group`` argument:
-                - ``CircuitNodeId``: return the ID in a CircuitNodeIds object if it belongs to
-                    the circuit.
-                - ``CircuitNodeIds``: return the IDs in a CircuitNodeIds object if they belong to
-                    the circuit.
-                - ``int``: if the node ID is present in all populations, returns a CircuitNodeIds
-                    object containing the corresponding node ID for all populations.
-                - ``sequence``: if all the values contained in the sequence are present in all
-                    populations, returns a CircuitNodeIds object containing the corresponding node
-                    IDs for all populations.
-                - ``str``: use a node set name as input. Returns a CircuitNodeIds object containing
-                    nodes selected by the node set.
-                - ``mapping``: Returns a CircuitNodeIds object containing nodes matching a
-                    properties filter.
-                - ``None``: return all node IDs of the circuit in a CircuitNodeIds object.
-            sample (int): If specified, randomly choose ``sample`` number of
-                IDs from the match result. If the size of the sample is greater than
-                the size of all the NodePopulations then all ids are taken and shuffled.
-            limit (int): If specified, return the first ``limit`` number of
-                IDs from the match result. If limit is greater than the size of all the populations,
-                all node IDs are returned.
+            circuit (bluepysnap.Circuit): the circuit object containing the node population
+            population_name (str): the name of the node population
 
         Returns:
-            CircuitNodeIds: returns a CircuitNodeIds containing all the node IDs and the
-                corresponding populations. All the explicitly requested IDs must be present inside
-                the circuit.
-
-        Raises:
-            BluepySnapError: when a population from a CircuitNodeIds is not present in the circuit.
-            BluepySnapError: when an id query via a int, sequence, or CircuitNodeIds is not present
-                in the circuit.
-
-        Examples:
-            The available group parameter values (example with 2 node populations pop1 and pop2):
-
-            >>> nodes = circuit.nodes
-            >>> nodes.ids(group=None)  #  returns all CircuitNodeIds from the circuit
-            >>> node_ids = CircuitNodeIds.from_arrays(["pop1", "pop2"], [1, 3])
-            >>> nodes.ids(group=node_ids)  #  returns ID 1 from pop1 and ID 3 from pop2
-            >>> nodes.ids(group=0)  #  returns CircuitNodeIds 0 from pop1 and pop2
-            >>> nodes.ids(group=[0, 1])  #  returns CircuitNodeIds 0 and 1 from pop1 and pop2
-            >>> nodes.ids(group="node_set_name")  # returns CircuitNodeIds matching node set
-            >>> nodes.ids(group={Node.LAYER: 2})  # returns CircuitNodeIds matching layer==2
-            >>> nodes.ids(group={Node.LAYER: [2, 3]})  # returns CircuitNodeIds with layer in [2,3]
-            >>> nodes.ids(group={Node.X: (0, 1)})  # returns CircuitNodeIds with 0 < x < 1
-            >>> # returns CircuitNodeIds matching one of the queries inside the 'or' list
-            >>> nodes.ids(group={'$or': [{ Node.LAYER: [2, 3]},
-            >>>                          { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
-            >>> # returns CircuitNodeIds matching all the queries inside the 'and' list
-            >>> nodes.ids(group={'$and': [{ Node.LAYER: [2, 3]},
-            >>>                           { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
+            NodePopulation: A NodePopulation object.
         """
-        if isinstance(group, CircuitNodeIds):
-            diff = np.setdiff1d(group.get_populations(unique=True), self.population_names)
-            if diff.size != 0:
-                raise BluepySnapError(f"Population {diff} does not exist in the circuit.")
-
-        fun = lambda x: (x.ids(group, raise_missing_property=False), x.name)
-        return self._get_ids_from_pop(fun, CircuitNodeIds, sample=sample, limit=limit)
-
-    def get(self, group=None, properties=None):   # pylint: disable=arguments-differ
-        """Node properties as a pandas DataFrame.
-
-        Args:
-            group (CircuitNodeIds/int/sequence/str/mapping/None): Which nodes will have their
-                properties returned depends on the type of the ``group`` argument:
-                See :py:class:`~bluepysnap.nodes.Nodes.ids`.
-
-            properties (str/list): If specified, return only the properties in the list.
-                Otherwise return all properties.
-
-        Returns:
-            pandas.DataFrame: Return a pandas DataFrame indexed by NodeCircuitIds containing the
-                properties from ``properties``.
-
-        Notes:
-            The NodePopulation.property_names function will give you all the usable properties
-            for the `properties` argument.
-        """
-        if properties is None:
-            properties = self.property_names
-        return super().get(group, properties)
-
-
-class NodeStorage:
-    """Node storage access."""
-
-    def __init__(self, config, circuit):
-        """Initializes a NodeStorage object from a node config and a Circuit.
-
-        Args:
-            config (dict): a node config from the global circuit config
-            circuit (bluepysnap.Circuit): the circuit object that contains the NodePopulation
-            from this storage.
-
-        Returns:
-            NodeStorage: A NodeStorage object.
-        """
-        self._h5_filepath = config['nodes_file']
-        self._csv_filepath = config.get('node_types_file')
-        self._populations_config = config.get('populations', {})
         self._circuit = circuit
-        self._populations = {}
+        self.name = population_name
 
     @property
-    def storage(self):
-        """Access to the libsonata node storage."""
-        return libsonata.NodeStorage(self._h5_filepath)
+    def _node_sets(self):
+        """Node sets defined for this node population."""
+        return self._circuit.node_sets
 
     @cached_property
-    def population_names(self):
-        """Returns all population names inside this file."""
-        return self.storage.population_names
-
-    @property
-    def h5_filepath(self):
-        """Returns the filepath of the Storage."""
-        return self._h5_filepath
-
-    @property
-    def csv_filepath(self):
-        """Returns the csv filepath of the Storage."""
-        return self._csv_filepath
-
-    @property
-    def circuit(self):
-        """Returns the circuit object containing this storage."""
-        return self._circuit
-
-    def population(self, population_name):
-        """Access the different populations from the storage."""
-        if population_name not in self._populations:
-            population_config = self._populations_config.get(population_name)
-            self._populations[population_name] = NodePopulation(
-                self, population_name, population_config=population_config)
-
-        return self._populations[population_name]
-
-    def load_population_data(self, population):
-        """Load node properties from SONATA Nodes.
-
-        Args:
-            population (str): a population name .
-
-        Returns:
-            pandas.DataFrame with node properties (zero-based index).
-        """
-        nodes = self.storage.open_population(population)
+    def _data(self):
+        """Collect data for the node population as a pandas.DataFrame."""
+        nodes = self._population
         categoricals = nodes.enumeration_names
 
-        node_count = nodes.size
-        result = pd.DataFrame(index=np.arange(node_count))
+        _all = nodes.select_all()
+        result = pd.DataFrame(index=np.arange(_all.flat_size))
 
-        _all = libsonata.Selection([(0, node_count)])
         for attr in sorted(nodes.attribute_names):
             if attr in categoricals:
                 enumeration = np.asarray(nodes.get_enumeration(attr, _all))
@@ -227,55 +79,23 @@ class NodeStorage:
             result[attr] = nodes.get_dynamics_attribute(attr.split(DYNAMICS_PREFIX)[1], _all)
         return result
 
-
-class NodePopulation:
-    """Node population access."""
-
-    def __init__(self, node_storage, population_name, population_config=None):
-        """Initializes a NodePopulation object from a NodeStorage and population name.
-
-        Args:
-            node_storage (NodeStorage): the node storage containing the node population
-            population_name (str): the name of the node population
-            population_config (dict): the config for the population
-
-        Returns:
-            NodePopulation: A NodePopulation object.
-        """
-        self._config = population_config or {}
-        self._node_storage = node_storage
-        self.name = population_name
+    @property
+    def _properties(self):
+        return self._circuit.to_libsonata.node_population_properties(self.name)
 
     @property
-    def _node_sets(self):
-        """Node sets defined for this node population."""
-        return self._node_storage.circuit.node_sets
-
-    @cached_property
-    def _data(self):
-        """Collected data for the node population as a pandas.DataFrame."""
-        return self._node_storage.load_population_data(self.name)
-
-    @cached_property
     def _population(self):
-        return self._node_storage.storage.open_population(self.name)
+        return self._circuit.to_libsonata.node_population(self.name)
 
     @cached_property
     def size(self):
         """Node population size."""
         return self._population.size
 
-    @cached_property
-    def config(self):
-        """Population config dictionary combined with the components dictionary."""
-        components = deepcopy(self._node_storage.circuit.config.get('components', {}))
-        components.update(self._config)
-        return components
-
     @property
     def type(self):
         """Population type."""
-        return self.config.get('type', DEFAULT_NODE_TYPE)
+        return self._properties.type
 
     @cached_property
     def _property_names(self):
@@ -292,8 +112,9 @@ class NodePopulation:
             set: a set containing the names of edge populations using this NodePopulation as
             source.
         """
-        return set(edge.name for edge in self._node_storage.circuit.edges.values() if
-                   self.name == edge.source.name)
+        return set(
+            edge.name for edge in self._circuit.edges.values() if self.name == edge.source.name
+        )
 
     def target_in_edges(self):
         """Set of edge population names that use this node population as target.
@@ -302,8 +123,9 @@ class NodePopulation:
             set: a set containing the names of edge populations using this NodePopulation as
             target.
         """
-        return set(edge.name for edge in self._node_storage.circuit.edges.values() if
-                   self.name == edge.target.name)
+        return set(
+            edge.name for edge in self._circuit.edges.values() if self.name == edge.target.name
+        )
 
     @property
     def property_names(self):
@@ -384,8 +206,10 @@ class NodePopulation:
             max_id = max(node_ids)
             min_id = min(node_ids)
         if min_id < 0 or max_id >= self._data.index.shape[0]:
-            raise BluepySnapError(f"All node IDs must be >= 0 and < {self._data.index.shape[0]} "
-                                  f"for population '{self.name}'")
+            raise BluepySnapError(
+                f"All node IDs must be >= 0 and < {self._data.index.shape[0]} "
+                f"for population '{self.name}'"
+            )
 
     def _check_property(self, prop):
         """Check if a property exists inside the dataset."""
@@ -500,8 +324,9 @@ class NodePopulation:
         if group is None:
             result = self._data.index.values
         elif isinstance(group, Mapping):
-            result = self._node_ids_by_filter(queries=group,
-                                              raise_missing_prop=raise_missing_property)
+            result = self._node_ids_by_filter(
+                queries=group, raise_missing_prop=raise_missing_property
+            )
         elif isinstance(group, np.ndarray):
             result = group
             self._check_ids(result)
@@ -513,8 +338,9 @@ class NodePopulation:
                 try:
                     result = [cid.id for cid in result if cid.population == self.name]
                 except AttributeError as e:
-                    raise BluepySnapError("All values from a list must be of type int or "
-                                          "CircuitNodeId.") from e
+                    raise BluepySnapError(
+                        "All values from a list must be of type int or CircuitNodeId."
+                    ) from e
             self._check_ids(result)
             preserve_order = isinstance(group, Sequence)
 
@@ -669,26 +495,26 @@ class NodePopulation:
                 Otherwise a pandas Series with rotation matrices indexed by node IDs.
         """
         # need to keep this quaternion ordering for quaternion2mat (expects w, x, y , z)
-        props = np.array([
-            Node.ORIENTATION_W,
-            Node.ORIENTATION_X,
-            Node.ORIENTATION_Y,
-            Node.ORIENTATION_Z
-        ])
+        props = np.array(
+            [Node.ORIENTATION_W, Node.ORIENTATION_X, Node.ORIENTATION_Y, Node.ORIENTATION_Z]
+        )
         props_mask = np.isin(props, list(self.property_names))
         orientation_count = np.count_nonzero(props_mask)
         if orientation_count == 4:
             trans = utils.quaternion2mat
         elif orientation_count in [1, 2, 3]:
             raise BluepySnapError(
-                "Missing orientation fields. Should be 4 quaternions or euler angles or nothing")
+                "Missing orientation fields. Should be 4 quaternions or euler angles or nothing"
+            )
         else:
             # need to keep this rotation_angle ordering for euler2mat (expects z, y, x)
-            props = np.array([
-                Node.ROTATION_ANGLE_Z,
-                Node.ROTATION_ANGLE_Y,
-                Node.ROTATION_ANGLE_X,
-            ])
+            props = np.array(
+                [
+                    Node.ROTATION_ANGLE_Z,
+                    Node.ROTATION_ANGLE_Y,
+                    Node.ROTATION_ANGLE_X,
+                ]
+            )
             props_mask = np.isin(props, list(self.property_names))
             trans = utils.euler2mat
 
@@ -703,7 +529,7 @@ class NodePopulation:
         args = [_get_values(prop) for prop in props]
         if utils.is_node_id(group):
             return trans(*args)[0]
-        return pd.Series(trans(*args), index=result.index, name='orientation')
+        return pd.Series(trans(*args), index=result.index, name="orientation")
 
     def count(self, group=None):
         """Total number of nodes for a given node group.
@@ -729,12 +555,141 @@ class NodePopulation:
         """Access to node morphologies."""
         from bluepysnap.morph import MorphHelper
 
-        return MorphHelper(self.config.get('morphologies_dir'),
-                           self,
-                           alternate_morphologies=self.config.get('alternate_morphologies'))
+        return MorphHelper(
+            self._properties.morphologies_dir,
+            self,
+            alternate_morphologies=self._properties.alternate_morphology_formats,
+        )
 
     @cached_property
     def models(self):
         """Access to node neuron models."""
         from bluepysnap.neuron_models import NeuronModelsHelper
-        return NeuronModelsHelper(self.config, self)
+
+        return NeuronModelsHelper(self._properties, self)
+
+    @cached_property
+    def h5_filepath(self):
+        """Get the H5 nodes file associated with population."""
+        for node_conf in self._circuit.config["networks"]["nodes"]:
+            if self.name in node_conf["populations"]:
+                return node_conf["nodes_file"]
+        raise BluepySnapError(f"h5_filepath not found for population '{self.name}'")
+
+
+class Nodes(
+    NetworkObject,
+    metaclass=AbstractDocSubstitutionMeta,
+    source_word="NetworkObject",
+    target_word="Node",
+):
+    """The top level Nodes accessor."""
+
+    _population_class = NodePopulation
+
+    def __init__(self, circuit):  # pylint: disable=useless-super-delegation
+        """Initialize the top level Nodes accessor."""
+        super().__init__(circuit)
+
+    @cached_property
+    def population_names(self):
+        """Defines all sorted node population names from the Circuit."""
+        return sorted(self._circuit.to_libsonata.node_populations)
+
+    def property_values(self, prop):
+        """Returns all the values for a given Nodes property."""
+        return set(
+            value
+            for pop in self.values()
+            if prop in pop.property_names
+            for value in pop.property_values(prop)
+        )
+
+    def ids(self, group=None, sample=None, limit=None):
+        """Returns the CircuitNodeIds corresponding to the nodes from ``group``.
+
+        Args:
+            group (CircuitNodeId/CircuitNodeIds/int/sequence/str/mapping/None): Which IDs will be
+            returned depends on the type of the ``group`` argument:
+                - ``CircuitNodeId``: return the ID in a CircuitNodeIds object if it belongs to
+                    the circuit.
+                - ``CircuitNodeIds``: return the IDs in a CircuitNodeIds object if they belong to
+                    the circuit.
+                - ``int``: if the node ID is present in all populations, returns a CircuitNodeIds
+                    object containing the corresponding node ID for all populations.
+                - ``sequence``: if all the values contained in the sequence are present in all
+                    populations, returns a CircuitNodeIds object containing the corresponding node
+                    IDs for all populations.
+                - ``str``: use a node set name as input. Returns a CircuitNodeIds object containing
+                    nodes selected by the node set.
+                - ``mapping``: Returns a CircuitNodeIds object containing nodes matching a
+                    properties filter.
+                - ``None``: return all node IDs of the circuit in a CircuitNodeIds object.
+            sample (int): If specified, randomly choose ``sample`` number of
+                IDs from the match result. If the size of the sample is greater than
+                the size of all the NodePopulations then all ids are taken and shuffled.
+            limit (int): If specified, return the first ``limit`` number of
+                IDs from the match result. If limit is greater than the size of all the populations,
+                all node IDs are returned.
+
+        Returns:
+            CircuitNodeIds: returns a CircuitNodeIds containing all the node IDs and the
+                corresponding populations. All the explicitly requested IDs must be present inside
+                the circuit.
+
+        Raises:
+            BluepySnapError: when a population from a CircuitNodeIds is not present in the circuit.
+            BluepySnapError: when an id query via a int, sequence, or CircuitNodeIds is not present
+                in the circuit.
+
+        Examples:
+            The available group parameter values (example with 2 node populations pop1 and pop2):
+
+            >>> nodes = circuit.nodes
+            >>> nodes.ids(group=None)  #  returns all CircuitNodeIds from the circuit
+            >>> node_ids = CircuitNodeIds.from_arrays(["pop1", "pop2"], [1, 3])
+            >>> nodes.ids(group=node_ids)  #  returns ID 1 from pop1 and ID 3 from pop2
+            >>> nodes.ids(group=0)  #  returns CircuitNodeIds 0 from pop1 and pop2
+            >>> nodes.ids(group=[0, 1])  #  returns CircuitNodeIds 0 and 1 from pop1 and pop2
+            >>> nodes.ids(group="node_set_name")  # returns CircuitNodeIds matching node set
+            >>> nodes.ids(group={Node.LAYER: 2})  # returns CircuitNodeIds matching layer==2
+            >>> nodes.ids(group={Node.LAYER: [2, 3]})  # returns CircuitNodeIds with layer in [2,3]
+            >>> nodes.ids(group={Node.X: (0, 1)})  # returns CircuitNodeIds with 0 < x < 1
+            >>> # returns CircuitNodeIds matching one of the queries inside the 'or' list
+            >>> nodes.ids(group={'$or': [{ Node.LAYER: [2, 3]},
+            >>>                          { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
+            >>> # returns CircuitNodeIds matching all the queries inside the 'and' list
+            >>> nodes.ids(group={'$and': [{ Node.LAYER: [2, 3]},
+            >>>                           { Node.X: (0, 1), Node.MTYPE: 'L1_SLAC' }]})
+        """
+        if isinstance(group, CircuitNodeIds):
+            diff = np.setdiff1d(group.get_populations(unique=True), self.population_names)
+            if diff.size != 0:
+                raise BluepySnapError(f"Population {diff} does not exist in the circuit.")
+
+        fun = lambda x: (x.ids(group, raise_missing_property=False), x.name)
+        return self._get_ids_from_pop(fun, CircuitNodeIds, sample=sample, limit=limit)
+
+    def get(self, group=None, properties=None):  # pylint: disable=arguments-differ
+        """Node properties as a pandas DataFrame.
+
+        Args:
+            group (CircuitNodeIds/int/sequence/str/mapping/None): Which nodes will have their
+                properties returned depends on the type of the ``group`` argument:
+                See :py:class:`~bluepysnap.nodes.Nodes.ids`.
+
+            properties (str/list): If specified, return only the properties in the list.
+                Otherwise return all properties.
+
+        Returns:
+            pandas.DataFrame: Return a pandas DataFrame indexed by NodeCircuitIds containing the
+                properties from ``properties``.
+
+        Notes:
+            The NodePopulation.property_names function will give you all the usable properties
+            for the `properties` argument.
+        """
+        if properties is None:
+            # not strictly needed, but ensure that the properties are always in the same order
+            properties = sorted(self.property_names)
+        return super().get(group, properties)
